/** \file
 * 
 * Sep 2, 2013
 *
 * Copyright Ian Kaplan 2013
 *
 * @author Ian Kaplan, www.bearcave.com, iank@bearcave.com
 */
package t9.trie;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.TreeSet;

import org.junit.Test;


/**
 * TrieBuilderTest
 * Sep 4, 2013
 * 
 * @author Ian Kaplan, iank@bearcave.com
 */
public class TrieBuilderTest {
    
    final static Pair[] A_words = new Pair[] {
        new Pair("a",5),
        new Pair("abandon",2201),
        new Pair("ability",783),
        new Pair("able",386),
        new Pair("abortion",1983),
        new Pair("about",179),
        new Pair("about",46),
        new Pair("above",1599),
        new Pair("above",896),
        new Pair("abroad",3392),
        new Pair("absence",2547),
        new Pair("absolute",3192),
        new Pair("absolutely",1152),
        new Pair("absorb",3145),
        new Pair("abstract",4045),
        new Pair("abuse",1548),
        new Pair("abuse",3778),
        new Pair("academic",1473),
        new Pair("accelerate",4977),
        new Pair("accent",4181),
        new Pair("accept",767),
        new Pair("acceptable",3445),
        new Pair("acceptance",3473),
        new Pair("access",1117),
        new Pair("access",4416),
        new Pair("accessible",4423),
        new Pair("accident",1720),
        new Pair("accommodate",3753),
        new Pair("accompany",2460),
        new Pair("accomplish",2366),
        new Pair("accomplishment",4351),
        new Pair("according",479),
        new Pair("account",1075),
        new Pair("account",1923),
        new Pair("accountability",4855),
        new Pair("accounting",3960),
        new Pair("accuracy",4111),
        new Pair("accurate",2754),
        new Pair("accurately",4950),
        new Pair("accusation",4924),
        new Pair("accuse",2041),
        new Pair("achieve",1155),
        new Pair("achievement",2038),
        new Pair("acid",3265),
        new Pair("acknowledge",1664),
        new Pair("acquire",2079),
        new Pair("acquisition",3893)
    };
    
    /**
     *  Test that the tree can be constructed and that it contains the expected elements
     */   
    @Test
    public void testTrieBuilder() {
        TreeSet<Pair> words = new TreeSet<Pair>();
        for (Pair word : A_words) {
            words.add( word );
        }
        TrieBuilder builder = new TrieBuilder();
        TrieNode A_root = builder.buildTrie(words);
        TreeSet<Pair> sortedList = builder.listTrie( A_root );
        assertTrue(words.equals(sortedList));
    }

    /**
     * Test that the Trie properly handles repeated characters.
     */
    @Test
    public void testAddLetter() {
        TrieBuilder builder = new TrieBuilder();
        String repeats = "aaaahhh";
        Pair wordPair = new Pair(repeats, 42 );
        TrieNode root = builder.addWord( null, wordPair );
        TreeSet<Pair> wordList = builder.listTrie(root);
        if (wordList != null && wordList.size() == 1) {
            Pair elem = wordList.first();
            if (elem != null) {
                String testWord = elem.word;
                assertTrue(repeats.equals(testWord));
            }
            else {
                fail("The word in the tree did not match");
            }
        }
        else {
            fail("The word list was either null or the wrong length");
        }
    }  // testAddLetter
    
}
